import {
  ExceptionFilter,
  Catch,
  ArgumentsHost,
  HttpException,
  HttpStatus,
  Logger,
} from '@nestjs/common';
import { error } from 'console';

// Catch any exception
@Catch()
export class AllExceptionsFilter implements ExceptionFilter {
  constructor(private readonly logger: Logger) {}
  catch(exception: unknown, host: ArgumentsHost) {
    const ctx = host.switchToHttp();
    const response = ctx.getResponse();
    const request = ctx.getRequest();

    const status =
      exception instanceof HttpException
        ? exception.getStatus()
        : HttpStatus.INTERNAL_SERVER_ERROR;

    const { body, url, method, originalUrl, rawHeaders } = request;
    let message = exception as string;
    let stack = undefined;

    if (exception instanceof Error) {
      message = exception.message;
      // trim the stack lines
      if (exception.stack) {
        stack = exception.stack.split('\n').slice(1, 3).join('\n');
      }
    }

    // For HTTP Exception, don't log the stack
    // if (exception instanceof HttpException) {
    //   stack = null;
    // }

    let logMsg: any;

    if (process.env.NODE_ENV === 'production') {
      logMsg = {
        message,
        request: {
          method,
          url,
          body,
          // originalUrl,
          rawHeaders,
        },
      };
      this.logger.error(logMsg, stack, AllExceptionsFilter.name);
    } else {
      const requstBody =
        Object.keys(body).length > 0 ? ` Body=${JSON.stringify(body)}` : '';

      const rawHeadersStr =
        Object.keys(rawHeaders).length > 0
          ? ` RawHeaders=${JSON.stringify(rawHeaders)}`
          : '';

      logMsg = `${message}\n${method.toLocaleUpperCase()} ${url}${requstBody}${rawHeadersStr}`;
      this.logger.error(logMsg, stack, AllExceptionsFilter.name);
    }

    response.header('Content-Type', 'application/json; charset=utf-8');
    response.status(status).json({
      statusCode: status,
      timestamp: new Date().toISOString(),
      path: request.url,
      error: message,
    });
  }
}
